@@ -23,6 +23,12 @@ double factorial_fast(double n)
 // [[Rcpp::export]]
 Rcpp::List irls_poisson_cpp_fast(arma::mat A, arma::vec b, double maxit, double tol)
 {
+  // // print out A
+  // Rcpp::Rcout << "A: " << A << std::endl;
+
+  // // print out b
+  // Rcpp::Rcout << "b: " << b << std::endl;
+
 //Def
   arma::vec x;
   x.zeros(A.n_cols,1);
@@ -47,17 +53,40 @@ Rcpp::List irls_poisson_cpp_fast(arma::mat A, arma::vec b, double maxit, double
   arma::vec z(nobs);
 
   for (int i = 0; i < maxit; ++i) {
+    // // print out i
+    // Rcpp::Rcout << "i: " << i << std::endl;
+
     eta = A * x;
+    // // print out eta
+    // Rcpp::Rcout << "eta: " << eta << std::endl;
 
     W = exp(eta);
+    // // print out W
+    // Rcpp::Rcout << "W: " << W << std::endl;
 
     z = eta+(b-W)/W;
+    // // print out z
+    // Rcpp::Rcout << "z: " << z << std::endl;
 
     xold = x;
+    // // print out xold
+    // Rcpp::Rcout << "xold: " << xold << std::endl;
 
     //coefficients
     varmatrix = A.t()*(W % A.each_col());
-    x = arma::solve(varmatrix, A.t()*(W % z), arma::solve_opts::likely_sympd);
+    // // print out varmatrix
+    // Rcpp::Rcout << "varmatrix: " << varmatrix << std::endl;
+
+    // if varmat has no nan solve it else raise warning and break
+    if (varmatrix.has_nan() || varmatrix.has_inf()) {
+      Rcpp::warning("irls_poisson_fast.cpp: varmatrix has nan. Stopping early at iteration %d", i);
+      break;
+    } else {
+      // Rcpp::Rcout << "varmatrix has no nan" << std::endl;
+      x = arma::solve(varmatrix, A.t()*(W % z), arma::solve_opts::likely_sympd);
+      // // print out x
+      // Rcpp::Rcout << "x: " << x << std::endl;
+    }
 
     if(sqrt(pow(arma::norm(x-xold), 2)) < tol){
       break;
